__version__ = '0.1.1'


from .Freezable import Freezable
from .Accession import Accession
from .Cohort    import Cohort
from .CloudData import CloudData


# Initialize the Minus80 Freezer
def _init():
    import os
    import apsw as lite
    from .Config import cf
    
    basedir = os.path.realpath(
        os.path.expanduser(cf.options.basedir)
    )
    
    # Create the basedir if not exists
    try:
        os.makedirs(basedir, exist_ok=True)
        os.makedirs(os.path.join(basedir, "databases"), exist_ok=True)
        os.makedirs(os.path.join(basedir, "tmp"), exist_ok=True)
        os.makedirs(os.path.join(basedir, "Raw"), exist_ok=True)
    except Exception:
        raise
    # Create the base database
#    lite.Connection(
#        os.path.join(basedir, 'databases', 'Minus80.Freezer.db')
#    ).cursor().execute('''
#        CREATE TABLE IF NOT EXISTS datasets (
#            name TEXT NOT NULL,
#            description TEXT,
#            type TEXT,
#            added datetime DEFAULT CURRENT_TIMESTAMP,
#            PRIMARY KEY(name, type)
#        );
#        INSERT OR IGNORE INTO datasets (name, description, type)
#        VALUES (?, ?, ?)''', ('Freezer', "Freezer Database", 'Minus80')
#    )   

_init()




class Accession(object):
    '''
    From google: Definition (noun): a new item added to an existing collection of
    books, paintings, or artifacts.  

    An Accession is an item that exists in an experimental collection. 
    
    Most of the time an accession is interoperable with a *sample*. However,
    the term sample can become confusing when an experiment has multiple
    samplings from the same sample, e.g. timecourse or different tissues. 
    '''

    def __init__(self,name,files=None,**kwargs):
        '''
        Create a new accession.

        Parameters
        ----------
        name : str
            The name of the accession
        files : iterable of str
            Files associated with the accession
        **kwargs : keyword arguments
            Any number of key=value arguments that
            contain metadata.

        Returns
        -------
        An accession object
        '''
        self.name = name
        if files != None:
            self.files = set(files)
        else:
            self.files = set()
        self.metadata = kwargs

    def __getitem__(self,key):
        '''
        Retrieve metadata about an accession. 

        Parameters
        ----------
        key : str

        Returns
        -------
        Value from the accession metadata corresponding
        to the key.
        '''
        return self.metadata[key]

    def __setitem__(self,key,val):
        '''
        Set metadata about an accession

        Parameters
        ----------
        key : str
            The metadata name
        val : str
            The value of the metadata
        '''
        self.metadata[key] = val

    def add_file(self,path):
        '''
        Add a file that is associated with the accession.

        Parameters
        ----------
        path: string

        Returns
        -------
        None
        '''
        self.files.add(path)

    def add_files(self,paths):
        '''
        Add multiple paths that are associated with an accession

        Parameters
        ----------
        paths : iterable of strings

        Returns
        -------
        None
        '''
        for path in paths:
            self.add_file(path)

    def __repr__(self): #pragma: no cover
        '''
        String representation of Accession
        '''
        return f'Accession({self.name},files={self.files},{self.metadata})'



#!/usr/bin/env python3

import sys
import argparse
import minus80 as m80
import click



@click.group()
@click.option('--debug/--no-debug', default=False)
def cli(debug):
    '''
    \b
        __  ____                  ____  ____ 
       /  |/  (_)___  __  _______( __ )/ __ \\
      / /|_/ / / __ \/ / / / ___/ __  / / / /
     / /  / / / / / / /_/ (__  ) /_/ / /_/ / 
    /_/  /_/_/_/ /_/\__,_/____/\____/\____/  
    
    Minus80 is a library for storing biological data. See minus80.linkage.io 
    for more details.
    '''
@cli.command(short_help='List the available minus80 datasets',
    help='Reports the available datasets **Frozen** in the minus80 database.'
)
@click.option('--name',  default='*', 
    help="The name of the dataset you want to check is available. The default value is the wildcard '*' which will return all available datasets with the specified dtype."
)
@click.option('--dtype', default='',
    help='Each dataset has a datatype associated with it. E.g.: `Cohort`. If no dtype is specified, all available dtypes  will be returned.'
)
def available(name,dtype):
    m80.Tools.available(name,dtype)
